#include "main.hpp"


// Init
int initialize() {
    // Initialize SDL
    if (!SDL_Init(SDL_INIT_VIDEO)) {
        std::cerr << "SDL could not initialize! SDL_Error: " << SDL_GetError() << std::endl;
        return 1; // Initialization failed
    }

    // Create a window with OpenGL context
    SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 3);
    SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 3);
    SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_CORE);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);
    SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 8);

    main_scale = SDL_GetDisplayContentScale(SDL_GetPrimaryDisplay());
    window = SDL_CreateWindow("Untitled", (int)(window_width * main_scale), (int)(window_height * main_scale), window_flags);
    if (!window) {
        std::cerr << "Window could not be created! SDL_Error: " << SDL_GetError() << std::endl;
        SDL_Quit();
        return 1; // Window creation failed
    }
    
    // Initialize OpenGL context
    SDL_GLContext gl_context = SDL_GL_CreateContext(window);
    if (!gl_context) {
        std::cerr << "OpenGL context could not be created! SDL_Error: " << SDL_GetError() << std::endl;
        SDL_DestroyWindow(window);
        SDL_Quit();
        return 1; // OpenGL context creation failed
    }

    SDL_GL_MakeCurrent(window, gl_context);
    SDL_GL_SetSwapInterval(1); // Enable VSync
    SDL_SetWindowPosition(window, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED);
    SDL_ShowWindow(window);

    // Initialize ImGui
    IMGUI_CHECKVERSION();
    ImGui::CreateContext();
    ImGuiIO& io = ImGui::GetIO(); (void)io;
    io.ConfigFlags |= ImGuiConfigFlags_NavEnableKeyboard; // Enable Keyboard Controls

    // Setup Dear ImGui style
    ImGui::StyleColorsDark();

    // Setup scale
    ImGuiStyle& style = ImGui::GetStyle();
    style.ScaleAllSizes(main_scale);
    style.FontScaleDpi = main_scale;

    // Setup Platform/Renderer bindings
    if (!ImGui_ImplSDL3_InitForOpenGL(window, gl_context)) {
        std::cerr << "ImGui_ImplSDL3_InitForOpenGL failed!" << std::endl;
        SDL_GL_DestroyContext(gl_context);
        SDL_DestroyWindow(window);
        SDL_Quit();
        return 1; // Initialization failed
    }

    if (!ImGui_ImplOpenGL3_Init("#version 330")) {
        std::cerr << "ImGui_ImplOpenGL3_Init failed!" << std::endl;
        ImGui_ImplSDL3_Shutdown();
        SDL_GL_DestroyContext(gl_context);
        SDL_DestroyWindow(window);
        SDL_Quit();
        return 1; // Initialization failed
    }

    return 0; // Initialization successful
}


// Cleanup
void cleanup() {
    // Shutdown ImGui
    ImGui_ImplOpenGL3_Shutdown();
    ImGui_ImplSDL3_Shutdown();
    ImGui::DestroyContext();

    // Destroy OpenGL context
    SDL_GLContext gl_context = SDL_GL_GetCurrentContext();
    if (gl_context) {
        SDL_GL_MakeCurrent(NULL, NULL);
        SDL_GL_DestroyContext(gl_context);
    }

    // Destroy window
    if (window) {
        SDL_DestroyWindow(window);
        window = nullptr;
    }

    // Quit SDL
    SDL_Quit();
    std::cout << "Cleanup successful." << std::endl;
}


// Main
int main(int argc, char* argv[]) {
    // Initialize
    if (initialize() != 0) {
        std::cerr << "Initialization failed." << std::endl;
        return -1; // Initialization failed
    }

    // Main loop
    running = true;
    last_time = SDL_GetTicks();
    while (running) {

        // Handle events
        while (SDL_PollEvent(&event)) {
            ImGui_ImplSDL3_ProcessEvent(&event);
            if (event.type == SDL_EVENT_QUIT) {
                running = false; // Exit loop on quit event
            }
        }

        // Start the ImGui frame
        ImGui_ImplOpenGL3_NewFrame();
        ImGui_ImplSDL3_NewFrame();
        ImGui::NewFrame();

        // Show a simple ImGui window
        ImGui::Begin("Hello, World!");
        ImGui::Text("This is a simple ImGui window.");
        if (ImGui::Button("Close")) {
            running = false; // Close button clicked
        }
        ImGui::End();

        // Display FPS in the ImGui window
        ImGui::Begin("FPS Counter");
        ImGui::Text("Current FPS: %.2f", fps);
        // Frame time graph
        ImGui::PlotLines("##FrameTime", frame_time_graph, 100, 0, "Frame Time (ms)", 0.0f, 0.1f, ImVec2(200, 80));
        ImGui::Text("Current Time: %u ms", current_time);

        ImGui::End();

        // Render
        ImGui::Render();
        glViewport(0, 0, (int)(window_width * main_scale), (int)(window_height * main_scale));
        glClearColor(0.45f, 0.55f, 0.60f, 1.00f);
        glClear(GL_COLOR_BUFFER_BIT);
        ImGui_ImplOpenGL3_RenderDrawData(ImGui::GetDrawData());
        SDL_GL_SwapWindow(window);

        // Calculate FPS
        current_time = SDL_GetTicks();
        frame_time = (current_time - last_time) / 1000.0f; // Convert milliseconds to seconds
        fps = 1.0f / frame_time;
        last_time = current_time;

        // Update frame time graph
        for (int i = 99; i > 0; --i) {
            frame_time_graph[i] = frame_time_graph[i - 1]; // Shift old values
        }
        frame_time_graph[0] = frame_time; // Add new frame time
    }

    // Cleanup
    cleanup();
}